@@ -1,7 +1,7 @@
 package SQL::Translator::Parser::Oracle;
 
 # -------------------------------------------------------------------
-# $Id: Oracle.pm,v 1.18 2004/04/22 19:56:28 kycl4rk Exp $
+# $Id: Oracle.pm,v 1.29 2007/03/19 22:32:31 duality72 Exp $
 # -------------------------------------------------------------------
 # Copyright (C) 2002-4 SQLFairy Authors
 #
@@ -49,7 +49,6 @@ From http://www.ss64.com/ora/table_c.html:
 
 tbl_defs:
    column datatype [DEFAULT expr] [column_constraint(s)]
-   table_constraint
    table_ref_constraint
 
 storage_options:
@@ -98,7 +97,7 @@ was altered to better handle the syntax created by DDL::Oracle.
 
 use strict;
 use vars qw[ $DEBUG $VERSION $GRAMMAR @EXPORT_OK ];
-$VERSION = sprintf "%d.%02d", q$Revision: 1.18 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Revision: 1.29 $ =~ /(\d+)\.(\d+)/;
 $DEBUG   = 0 unless defined $DEBUG;
 
 use Data::Dumper;
@@ -113,11 +112,9 @@ $::RD_ERRORS = 1; # Make sure the parser dies when it encounters an error
 $::RD_WARN   = 1; # Enable warnings. This will warn on unused rules &c.
 $::RD_HINT   = 1; # Give out hints to help fix problems.
 
-my $parser; 
+$GRAMMAR = q`
 
-$GRAMMAR = q!
-
-{ my ( %tables, %indices, %constraints, $table_order, @table_comments ) }
+{ my ( %tables, %indices, %constraints, $table_order, @table_comments, %views, $view_order, %procedures, $proc_order ) }
 
 #
 # The "eofile" rule makes the parser fail if any "statement" rule
@@ -131,12 +128,15 @@ startrule : statement(s) eofile
             tables      => \%tables,
             indices     => \%indices,
             constraints => \%constraints,
+            views       => \%views,
+            procedures  => \%procedures,
         };
     }
 
 eofile : /^\Z/
 
 statement : remark
+	| run
     | prompt
     | create
     | table_comment
@@ -196,29 +196,77 @@ create : create_table table_name '(' create_definition(s /,/) ')' table_option(s
         1;
     }
 
-create : /create/i UNIQUE(?) /index/i WORD /on/i table_name parens_word_list table_option(?) ';'
+create : create_index index_name /on/i table_name index_expr table_option(?) ';'
     {
-        my $table_name = $item[6];
-        if ( $item[2] ) {
+        my $table_name = $item[4];
+        if ( $item[1] ) {
             push @{ $constraints{ $table_name } }, {
-                name   => $item[4],
+                name   => $item[2],
                 type   => 'unique',
-                fields => $item[7][0],
+                fields => $item[5],
             };
         }
         else {
             push @{ $indices{ $table_name } }, {
-                name   => $item[4],
+                name   => $item[2],
                 type   => 'normal',
-                fields => $item[7][0],
+                fields => $item[5],
             };
         }
     }
 
+index_expr: parens_word_list
+	{ $item[1] }
+	| '(' WORD parens_word_list ')'
+	{
+		my $arg_list = join(",", @{$item[3]});
+		$return = "$item[2]($arg_list)";
+	}
+
+create : /create/i /or replace/i /procedure/i table_name not_end m#^/$#im
+	{
+		@table_comments = ();
+        my $proc_name = $item[4];
+        # Hack to strip owner from procedure name
+        $proc_name =~ s#.*\.##;
+        my $owner = '';
+        my $sql = "$item[1] $item[2] $item[3] $item[4] $item[5]";
+        
+        $procedures{ $proc_name }{'order'}  = ++$proc_order;
+        $procedures{ $proc_name }{'name'}   = $proc_name;
+        $procedures{ $proc_name }{'owner'}  = $owner;
+        $procedures{ $proc_name }{'sql'}    = $sql;
+	}
+
+not_end: m#.*?(?=^/$)#ism
+
+create : /create/i /or replace/i /force/i /view/i table_name not_delimiter ';'
+	{
+		@table_comments = ();
+        my $view_name = $item[5];
+        # Hack to strip owner from view name
+        $view_name =~ s#.*\.##;
+        my $sql = "$item[1] $item[2] $item[3] $item[4] $item[5] $item[6] $item[7]";
+        
+        $views{ $view_name }{'order'}  = ++$view_order;
+        $views{ $view_name }{'name'}   = $view_name;
+        $views{ $view_name }{'sql'}    = $sql;
+	}
+
+not_delimiter: /.*?(?=;)/is
+
 # Create anything else (e.g., domain, function, etc.)
-create : /create/i WORD /[^;]+/ ';'
+create : ...!create_table ...!create_index /create/i WORD /[^;]+/ ';'
     { @table_comments = () }
 
+create_index : /create/i UNIQUE(?) /index/i
+	{ $return = @{$item[2]} }
+
+index_name : NAME '.' NAME
+    { $item[3] }
+    | NAME 
+    { $item[1] }
+
 global_temporary: /global/i /temporary/i
 
 table_name : NAME '.' NAME
@@ -254,6 +302,8 @@ comment : /\/\*/ /[^\*]+/ /\*\//
 
 remark : /^REM\s+.*\n/
 
+run : /^(RUN|\/)\s+.*\n/
+
 prompt : /prompt/i /(table|index|sequence|trigger)/i ';'
 
 prompt : /prompt\s+create\s+.*\n/i
@@ -319,15 +369,21 @@ field : comment(s?) field_name data_type field_meta(s?) comment(s?)
 
 field_name : NAME
 
-data_type : ora_data_type parens_value_list(?)
+data_type : ora_data_type data_size(?)
     { 
         $return  = { 
             type => $item[1],
             size => $item[2][0] || '',
         } 
     }
+    
+data_size : '(' VALUE(s /,/) data_size_modifier(?) ')'
+    { $item[2] } 
 
-column_constraint : constraint_name(?) column_constraint_type 
+data_size_modifier: /byte/i
+	| /char/i
+
+column_constraint : constraint_name(?) column_constraint_type constraint_state(s?)
     {
         my $desc       = $item{'column_constraint_type'};
         my $type       = $desc->{'type'};
@@ -344,41 +400,39 @@ column_constraint : constraint_name(?) column_constraint_type
             reference_table  => $desc->{'reference_table'},
             reference_fields => $desc->{'reference_fields'},
 #            match_type       => $desc->{'match_type'},
-#            on_update_do     => $desc->{'on_update_do'},
+#            on_update        => $desc->{'on_update'},
         } 
     }
 
 constraint_name : /constraint/i NAME { $item[2] }
 
-column_constraint_type : /not null/i { $return = { type => 'not_null' } }
-    | /null/ 
-        { $return = { type => 'null' } }
-    | /unique/ 
+column_constraint_type : /not\s+null/i { $return = { type => 'not_null' } }
+    | /unique/i
         { $return = { type => 'unique' } }
-    | /primary key/i 
+    | /primary\s+key/i 
         { $return = { type => 'primary_key' } }
     | /check/i '(' /[^)]+/ ')' 
-        { $return = { type => 'check', expression => $item[2] } }
-    | /references/i table_name parens_word_list(?) on_delete_do(?) 
+        { $return = { type => 'check', expression => $item[3] } }
+    | /references/i table_name parens_word_list(?) on_delete(?) 
     {
         $return              =  {
             type             => 'foreign_key',
             reference_table  => $item[2],
             reference_fields => $item[3][0],
 #            match_type       => $item[4][0],
-            on_delete_do     => $item[5][0],
+            on_delete     => $item[5][0],
         }
     }
 
-#constraint_state : deferrable { $return = { type => $item[1] } }
-#    | deferred { $return = { type => $item[1] } }
-#    | /(no)?rely/ { $return = { type => $item[1] } }
+constraint_state : deferrable { $return = { type => $item[1] } }
+    | deferred { $return = { type => $item[1] } }
+    | /(no)?rely/i { $return = { type => $item[1] } }
 #    | /using/i /index/i using_index_clause 
-#        { $return = { type => 'using_index', index => $item[3] }
-#    | (dis)?enable { $return = { type => $item[1] } }
-#    | (no)?validate { $return = { type => $item[1] } }
-#    | /exceptions/i /into/i table_name 
-#        { $return = { type => 'exceptions_into', table => $item[3] } }
+#        { $return = { type => 'using_index', index => $item[3] } }
+    | /(dis|en)able/i { $return = { type => $item[1] } }
+    | /(no)?validate/i { $return = { type => $item[1] } }
+    | /exceptions/i /into/i table_name 
+        { $return = { type => 'exceptions_into', table => $item[3] } }
 
 deferrable : /not/i /deferrable/i 
     { $return = 'not_deferrable' }
@@ -400,13 +454,13 @@ ora_data_type :
     |
     /(pls_integer|binary_integer)/i { $return = 'integer' }
     |
-    /interval\s+day/i { $return = 'interval_day' }
+    /interval\s+day/i { $return = 'interval day' }
     |
-    /interval\s+year/i { $return = 'interval_year' }
+    /interval\s+year/i { $return = 'interval year' }
     |
-    /long\s+raw/i { $return = 'long_raw' }
+    /long\s+raw/i { $return = 'long raw' }
     |
-    /(long|date|timestamp|raw|rowid|urowid|mlslabel|clob|nclob|blob|bfile)/i { $item[1] }
+    /(long|date|timestamp|raw|rowid|urowid|mlslabel|clob|nclob|blob|bfile|float)/i { $item[1] }
 
 parens_value_list : '(' VALUE(s /,/) ')'
     { $item[2] }
@@ -427,6 +481,14 @@ default_val  : /default/i /(?:')?[\w\d.-]*(?:')?/
             value     => $val,
         }
     }
+    | /null/i
+    {
+        $return =  {
+            supertype => 'constraint',
+            type      => 'default',
+            value     => 'NULL',
+        }
+    }
 
 create_table : /create/i global_temporary(?) /table/i
 
@@ -452,7 +514,7 @@ key_value : WORD VALUE
 
 table_option : /[^;]+/
 
-table_constraint : comment(s?) constraint_name(?) table_constraint_type deferrable(?) deferred(?) comment(s?)
+table_constraint : comment(s?) constraint_name(?) table_constraint_type deferrable(?) deferred(?) constraint_state(s?) comment(s?)
     {
         my $desc       = $item{'table_constraint_type'};
         my $type       = $desc->{'type'};
@@ -471,13 +533,13 @@ table_constraint : comment(s?) constraint_name(?) table_constraint_type deferrab
             reference_table  => $desc->{'reference_table'},
             reference_fields => $desc->{'reference_fields'},
 #            match_type       => $desc->{'match_type'}[0],
-            on_delete_do     => $desc->{'on_delete_do'},
-            on_update_do     => $desc->{'on_update_do'},
+            on_delete        => $desc->{'on_delete'} || $desc->{'on_delete_do'},
+            on_update        => $desc->{'on_update'} || $desc->{'on_update_do'},
             comments         => [ @comments ],
         } 
     }
 
-table_constraint_type : /primary key/i '(' NAME(s /,/) ')' 
+table_constraint_type : /primary key/i '(' NAME(s /,/) ')'
     { 
         $return = {
             type   => 'primary_key',
@@ -501,21 +563,21 @@ table_constraint_type : /primary key/i '(' NAME(s /,/) ')'
         }
     }
     |
-    /foreign key/i '(' NAME(s /,/) ')' /references/i table_name parens_word_list(?) on_delete_do(?)
+    /foreign key/i '(' NAME(s /,/) ')' /references/i table_name parens_word_list(?) on_delete(?)
     {
         $return              =  {
             type             => 'foreign_key',
             fields           => $item[3],
             reference_table  => $item[6],
             reference_fields => $item[7][0],
-            match_type       => $item[8][0],
-            on_delete_do     => $item[9][0],
-            on_update_do     => $item[10][0],
+#            match_type       => $item[8][0],
+            on_delete     => $item[8][0],
+#            on_update     => $item[9][0],
         }
     }
 
-on_delete_do : /on delete/i WORD(s)
-    { $item[2] }
+on_delete : /on delete/i WORD(s)
+    { join(' ', @{$item[2]}) }
 
 UNIQUE : /unique/i { $return = 1 }
 
@@ -532,12 +594,12 @@ VALUE   : /[-+]?\.?\d+(?:[eE]\d+)?/
     | /NULL/
     { 'NULL' }
 
-!;
+`;
 
 # -------------------------------------------------------------------
 sub parse {
     my ( $translator, $data ) = @_;
-    $parser ||= Parse::RecDescent->new($GRAMMAR);
+    my $parser = Parse::RecDescent->new($GRAMMAR);
 
     local $::RD_TRACE = $translator->trace ? 1 : undef;
     local $DEBUG      = $translator->debug;
@@ -549,7 +611,9 @@ sub parse {
 
     my $result = $parser->startrule( $data );
     die "Parse failed.\n" unless defined $result;
-    warn Dumper($result) if $DEBUG;
+    if ( $DEBUG ) {
+        warn "Parser results =\n", Dumper($result), "\n";
+    }
 
     my $schema      = $translator->schema;
     my $indices     = $result->{'indices'};
@@ -587,12 +651,6 @@ sub parse {
                 is_nullable       => $fdata->{'null'},
                 comments          => $fdata->{'comments'},
             ) or die $table->error;
-
-            for my $cdata ( @{ $fdata->{'constraints'} } ) {
-                next unless $cdata->{'type'} eq 'foreign_key';
-                $cdata->{'fields'} ||= [ $field->name ];
-                push @{ $tdata->{'constraints'} }, $cdata;
-            }
         }
 
         push @{ $tdata->{'indices'} }, @{ $indices->{ $table_name } || [] };
@@ -615,11 +673,32 @@ sub parse {
                 reference_table  => $cdata->{'reference_table'},
                 reference_fields => $cdata->{'reference_fields'},
                 match_type       => $cdata->{'match_type'} || '',
-                on_delete        => $cdata->{'on_delete_do'},
-                on_update        => $cdata->{'on_update_do'},
+                on_delete        => $cdata->{'on_delete'} || $cdata->{'on_delete_do'},
+                on_update        => $cdata->{'on_update'} || $cdata->{'on_update_do'},
             ) or die $table->error;
         }
     }
+    
+    my @procedures = sort { 
+        $result->{procedures}->{ $a }->{'order'} <=> $result->{procedures}->{ $b }->{'order'}
+    } keys %{ $result->{procedures} };
+    foreach my $proc_name (@procedures) {
+    	$schema->add_procedure(
+    		name  => $proc_name,
+    		owner => $result->{procedures}->{$proc_name}->{owner},
+    		sql   => $result->{procedures}->{$proc_name}->{sql},
+		);
+    }
+
+    my @views = sort { 
+        $result->{views}->{ $a }->{'order'} <=> $result->{views}->{ $b }->{'order'}
+    } keys %{ $result->{views} };
+    foreach my $view_name (keys %{ $result->{views} }) {
+    	$schema->add_view(
+    		name => $view_name,
+    		sql  => $result->{views}->{$view_name}->{sql},
+		);
+    }
 
     return 1;
 }
