@@ -1,7 +1,7 @@
 package SQL::Translator::Schema::Index;
 
 # ----------------------------------------------------------------------
-# $Id: Index.pm,v 1.8 2004/02/09 22:15:15 kycl4rk Exp $
+# $Id: Index.pm,v 1.17 2007/03/06 23:50:23 duality72 Exp $
 # ----------------------------------------------------------------------
 # Copyright (C) 2002-4 SQLFairy Authors
 #
@@ -46,14 +46,14 @@ Primary and unique keys are table constraints, not indices.
 =cut
 
 use strict;
-use Class::Base;
 use SQL::Translator::Schema::Constants;
 use SQL::Translator::Utils 'parse_list_arg';
 
-use base 'Class::Base';
+use base 'SQL::Translator::Schema::Object';
+
 use vars qw($VERSION $TABLE_COUNT $VIEW_COUNT);
 
-$VERSION = sprintf "%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Revision: 1.17 $ =~ /(\d+)\.(\d+)/;
 
 my %VALID_INDEX_TYPE = (
     UNIQUE,    1,
@@ -62,7 +62,10 @@ my %VALID_INDEX_TYPE = (
 );
 
 # ----------------------------------------------------------------------
-sub init {
+
+__PACKAGE__->_attributes( qw/
+    name type fields table
+/);
 
 =pod
 
@@ -74,16 +77,6 @@ Object constructor.
 
 =cut
 
-    my ( $self, $config ) = @_;
-
-    for my $arg ( qw[ name type fields table ] ) {
-        next unless $config->{ $arg };
-        defined $self->$arg( $config->{ $arg } ) or return;
-    }
-
-    return $self;
-}
-
 # ----------------------------------------------------------------------
 sub fields {
 
@@ -240,6 +233,49 @@ Get or set the index's type.
     return $self->{'type'} || NORMAL;
 }
 
+# ----------------------------------------------------------------------
+sub equals {
+
+=pod
+
+=head2 equals
+
+Determines if this index is the same as another
+
+  my $isIdentical = $index1->equals( $index2 );
+
+=cut
+
+    my $self = shift;
+    my $other = shift;
+    my $case_insensitive = shift;
+    my $ignore_index_names = shift;
+    
+    return 0 unless $self->SUPER::equals($other);
+    unless ($ignore_index_names) {
+        return 0 unless $case_insensitive ? uc($self->name) eq uc($other->name) : $self->name eq $other->name;
+    }
+    #return 0 unless $self->is_valid eq $other->is_valid;
+    return 0 unless $self->type eq $other->type;
+    
+    # Check fields, regardless of order
+    my %otherFields = ();	# create a hash of the other fields
+    foreach my $otherField ($other->fields) {
+    	$otherField = uc($otherField) if $case_insensitive;
+    	$otherFields{$otherField} = 1;
+    }
+    foreach my $selfField ($self->fields) { # check for self fields in hash
+    	$selfField = uc($selfField) if $case_insensitive;
+    	return 0 unless $otherFields{$selfField};
+    	delete $otherFields{$selfField};
+    }
+    # Check all other fields were accounted for
+    return 0 unless keys %otherFields == 0;
+
+    return 0 unless $self->_compare_objects(scalar $self->options, scalar $other->options);
+    return 0 unless $self->_compare_objects(scalar $self->extra, scalar $other->extra);
+    return 1;
+}
 
 # ----------------------------------------------------------------------
 sub DESTROY {
