@@ -1,7 +1,7 @@
 package SQL::Translator::Schema::Procedure;
 
 # ----------------------------------------------------------------------
-# $Id: Procedure.pm,v 1.2 2004/02/09 22:15:15 kycl4rk Exp $
+# $Id: Procedure.pm,v 1.8 2007/03/21 15:20:50 duality72 Exp $
 # ----------------------------------------------------------------------
 # Copyright (C) 2002-4 SQLFairy Authors
 #
@@ -48,16 +48,19 @@ stored procedures (and possibly other pieces of nameable SQL code?).
 =cut
 
 use strict;
-use Class::Base;
 use SQL::Translator::Utils 'parse_list_arg';
 
-use base 'Class::Base';
+use base 'SQL::Translator::Schema::Object';
+
 use vars qw($VERSION);
 
-$VERSION = sprintf "%d.%02d", q$Revision: 1.2 $ =~ /(\d+)\.(\d+)/;
+$VERSION = sprintf "%d.%02d", q$Revision: 1.8 $ =~ /(\d+)\.(\d+)/;
 
 # ----------------------------------------------------------------------
-sub init {
+
+__PACKAGE__->_attributes( qw/
+    name sql parameters comments owner sql schema order
+/);
 
 =pod
 
@@ -69,16 +72,6 @@ Object constructor.
 
 =cut
 
-    my ( $self, $config ) = @_;
-
-    for my $arg ( qw[ name sql parameters comments owner sql schema ] ) {
-        next unless $config->{ $arg };
-        $self->$arg( $config->{ $arg } ) or return;
-    }
-
-    return $self;
-}
-
 # ----------------------------------------------------------------------
 sub parameters {
 
@@ -112,7 +105,7 @@ Gets and set the parameters of the stored procedure.
         $self->{'parameters'} = \@unique;
     }
 
-    return wantarray ? @{ $self->{'parameters'} || [] } : $self->{'parameters'};
+    return wantarray ? @{ $self->{'parameters'} || [] } : ($self->{'parameters'} || '');
 }
 
 # ----------------------------------------------------------------------
@@ -247,6 +240,46 @@ Get or set the procedures's schema object.
     return $self->{'schema'};
 }
 
+# ----------------------------------------------------------------------
+sub equals {
+
+=pod
+
+=head2 equals
+
+Determines if this procedure is the same as another
+
+  my $isIdentical = $procedure1->equals( $procedure2 );
+
+=cut
+
+    my $self = shift;
+    my $other = shift;
+    my $case_insensitive = shift;
+    my $ignore_sql = shift;
+    
+    return 0 unless $self->SUPER::equals($other);
+    return 0 unless $case_insensitive ? uc($self->name) eq uc($other->name) : $self->name eq $other->name;
+    
+    unless ($ignore_sql) {
+        my $selfSql = $self->sql;
+        my $otherSql = $other->sql;
+        # Remove comments
+        $selfSql =~ s/--.*$//mg;
+        $otherSql =~ s/--.*$//mg;
+        # Collapse whitespace to space to avoid whitespace comparison issues
+        $selfSql =~ s/\s+/ /sg;
+        $otherSql =~ s/\s+/ /sg;
+        return 0 unless $selfSql eq $otherSql;
+    }
+    
+    return 0 unless $self->_compare_objects(scalar $self->parameters, scalar $other->parameters);
+#    return 0 unless $self->comments eq $other->comments;
+#    return 0 unless $case_insensitive ? uc($self->owner) eq uc($other->owner) : $self->owner eq $other->owner;
+    return 0 unless $self->_compare_objects(scalar $self->extra, scalar $other->extra);
+    return 1;
+}
+
 # ----------------------------------------------------------------------
 sub DESTROY {
     my $self = shift;
